"""
UI definitions service implementation.
"""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
from typing import Any

# Third-party imports
from PySide6.QtCore import QEvent, QSize, Qt, QTimer
from PySide6.QtGui import QColor
from PySide6.QtWidgets import QGraphicsDropShadowEffect, QSizeGrip

# Local imports
from ...widgets.custom_grips.custom_grips import CustomGrip
from ..settings import get_settings_service
from .window_service import WindowService


# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////
class UiDefinitionsService:
    """Service responsible for main window UI setup and grip resizing."""

    @staticmethod
    def apply_definitions(window: Any) -> None:
        """Configure all UI elements and connect base window events."""
        settings_service = get_settings_service()
        app_settings = settings_service.app

        def double_click_maximize_restore(event: Any) -> None:
            if event.type() == QEvent.Type.MouseButtonDblClick:
                QTimer.singleShot(250, lambda: WindowService.maximize_restore(window))

        window.ui.headerContainer.mouseDoubleClickEvent = double_click_maximize_restore

        if app_settings.NAME:
            window.ui.headerContainer.set_app_name(app_settings.NAME)
        if app_settings.DESCRIPTION:
            window.ui.headerContainer.set_app_description(app_settings.DESCRIPTION)
        if app_settings.APP_WIDTH and app_settings.APP_HEIGHT:
            window.resize(app_settings.APP_WIDTH, app_settings.APP_HEIGHT)
        if app_settings.APP_MIN_SIZE:
            window.setMinimumSize(QSize(*app_settings.APP_MIN_SIZE))

        if app_settings.ENABLE_CUSTOM_TITLE_BAR:
            window.setWindowFlags(Qt.WindowType.FramelessWindowHint)
            window.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground)

            def move_window(event: Any) -> None:
                if WindowService.get_status():
                    WindowService.maximize_restore(window)
                if event.buttons() == Qt.MouseButton.LeftButton:
                    window.move(window.pos() + event.globalPos() - window.dragPos)
                    window.dragPos = event.globalPos()
                    event.accept()

            window.ui.headerContainer.mouseMoveEvent = move_window

            window.left_grip = CustomGrip(window, Qt.Edge.LeftEdge, True)
            window.right_grip = CustomGrip(window, Qt.Edge.RightEdge, True)
            window.top_grip = CustomGrip(window, Qt.Edge.TopEdge, True)
            window.bottom_grip = CustomGrip(window, Qt.Edge.BottomEdge, True)
        else:
            window.ui.appMargins.setContentsMargins(0, 0, 0, 0)
            window.ui.headerContainer.minimizeAppBtn.hide()
            window.ui.headerContainer.maximizeRestoreAppBtn.hide()
            window.ui.headerContainer.closeAppBtn.hide()
            window.ui.bottomBar.appSizeGrip.hide()

        window.shadow = QGraphicsDropShadowEffect(window)
        window.shadow.setBlurRadius(17)
        window.shadow.setXOffset(0)
        window.shadow.setYOffset(0)
        window.shadow.setColor(QColor(0, 0, 0, 150))
        window.ui.bgApp.setGraphicsEffect(window.shadow)

        window.sizegrip = QSizeGrip(window.ui.bottomBar.appSizeGrip)
        window.sizegrip.setStyleSheet(
            "width: 20px; height: 20px; margin 0px; padding: 0px;"
        )

        window.ui.headerContainer.minimizeAppBtn.clicked.connect(
            lambda: window.showMinimized()
        )
        window.ui.headerContainer.maximizeRestoreAppBtn.clicked.connect(
            lambda: WindowService.maximize_restore(window)
        )
        window.ui.headerContainer.closeAppBtn.clicked.connect(lambda: window.close())

    @staticmethod
    def resize_grips(window: Any) -> None:
        """Resize custom grips when the main window size changes."""
        if get_settings_service().app.ENABLE_CUSTOM_TITLE_BAR:
            window.left_grip.setGeometry(0, 10, 10, window.height())
            window.right_grip.setGeometry(window.width() - 10, 10, 10, window.height())
            window.top_grip.setGeometry(0, 0, window.width(), 10)
            window.bottom_grip.setGeometry(0, window.height() - 10, window.width(), 10)



# ///////////////////////////////////////////////////////////////
# SERVICES.BOOTSTRAP - One-shot application boot sequence
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Bootstrap services — system configuration and initialization sequence."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
from pathlib import Path
from typing import Any

from .contracts import InitOptions, InitStep, OverwritePolicy, StepStatus
from .exceptions import BootstrapError, InitAlreadyInitializedError, InitStepError
from .init_service import InitService
from .initializer import Initializer
from .sequence import InitializationSequence
from .startup_config import StartupConfig

_init_service = InitService()


# ///////////////////////////////////////////////////////////////
# PUBLIC HELPERS
# ///////////////////////////////////////////////////////////////
def init(
    mk_theme: bool = True,
    verbose: bool = True,
    project_root: str | None = None,
    bin_path: str | None = None,
    overwrite_policy: OverwritePolicy = OverwritePolicy.ASK,
    mk_config: bool = True,
    mk_translations: bool = True,
    build_resources: bool = True,
    generate_main: bool = False,
) -> dict[str, Any]:
    """Initialize the EzQt_App application (boot sequence).

    Parameters
    ----------
    mk_theme:
        Generate theme file (default: ``True``).
    verbose:
        Show progress output (default: ``True``).

    Returns
    -------
    dict
        Boot sequence summary.
    """
    options = InitOptions(
        project_root=None if project_root is None else Path(project_root),
        bin_path=None if bin_path is None else Path(bin_path),
        mk_theme=mk_theme,
        mk_config=mk_config,
        mk_translations=mk_translations,
        build_resources=build_resources,
        generate_main=generate_main,
        verbose=verbose,
        overwrite_policy=overwrite_policy,
    )
    return _init_service.run(options).to_dict()


def setup_project(base_path: str | None = None) -> bool:
    """Scaffold a new EzQt_App project directory.

    Parameters
    ----------
    base_path:
        Target directory (defaults to current working directory).
    """
    from pathlib import Path

    from ..application.file_service import FileService

    return FileService(Path(base_path) if base_path else None).setup_project()


def generate_assets() -> bool:
    """Generate all required asset files."""
    from ..application.file_service import FileService

    return FileService().generate_all_assets()


def configure_startup() -> None:
    """Run system-level startup configuration (encoding, locale, env)."""
    StartupConfig().configure()


# ///////////////////////////////////////////////////////////////
# PUBLIC API
# ///////////////////////////////////////////////////////////////
__all__ = [
    "Initializer",
    "InitOptions",
    "OverwritePolicy",
    "InitService",
    "StartupConfig",
    "InitializationSequence",
    "InitStep",
    "StepStatus",
    "BootstrapError",
    "InitAlreadyInitializedError",
    "InitStepError",
    "init",
    "setup_project",
    "generate_assets",
    "configure_startup",
]



# ///////////////////////////////////////////////////////////////
# SERVICES.TRANSLATION.SERVICE - Translation service
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Translation service implementation."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Local imports
from ...domain.ports.translation_service import TranslationServiceProtocol
from .manager import get_translation_manager


# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////
class TranslationService(TranslationServiceProtocol):
    """Service wrapper around translation management."""

    def change_language(self, language_name: str) -> bool:
        """Switch application language using its display name."""
        return get_translation_manager().load_language(language_name)

    def change_language_by_code(self, language_code: str) -> bool:
        """Switch application language using ISO code (e.g. ``"fr"``)."""
        return get_translation_manager().load_language_by_code(language_code)

    def get_available_languages(self) -> list[str]:
        """Return available language codes."""
        return get_translation_manager().get_available_languages()

    def get_current_language_name(self) -> str:
        """Return the current language display name."""
        return get_translation_manager().get_current_language_name()

    def get_current_language_code(self) -> str:
        """Return the current language code."""
        return get_translation_manager().get_current_language_code()

    def translate(self, text: str) -> str:
        """Translate a text using the active language context."""
        return get_translation_manager().translate(text)


# ///////////////////////////////////////////////////////////////
# SINGLETONS
# ///////////////////////////////////////////////////////////////
_translation_service = TranslationService()


# ///////////////////////////////////////////////////////////////
# FUNCTIONS
# ///////////////////////////////////////////////////////////////
def get_translation_service() -> TranslationService:
    """Return the singleton translation service."""
    return _translation_service



# ///////////////////////////////////////////////////////////////
# TESTS.UNIT.TEST_SERVICES.TEST_APP_FUNCTIONS - Config service tests
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Unit tests for v5 configuration and resource services."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
from pathlib import Path

# Third-party imports
import yaml

# Local imports
from ezqt_app.services.config.config_service import ConfigService, get_package_resource

# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////


class TestConfigServiceV5:
    """Tests aligned with v5 service architecture."""

    def setup_method(self) -> None:
        self.service = ConfigService()

    def test_load_config_success_from_project_root(self, tmp_path: Path) -> None:
        config_dir = tmp_path / "bin" / "config"
        config_dir.mkdir(parents=True, exist_ok=True)

        config_file = config_dir / "app.config.yaml"
        config_data = {
            "app": {
                "name": "Test App",
                "description": "Test Description",
                "theme": "dark",
            }
        }
        with open(config_file, "w", encoding="utf-8") as file:
            yaml.safe_dump(config_data, file)

        self.service.set_project_root(tmp_path)
        loaded = self.service.load_config("app")

        assert loaded["app"]["name"] == "Test App"
        assert loaded["app"]["theme"] == "dark"

    def test_load_config_missing_returns_empty_dict(self, tmp_path: Path) -> None:
        self.service.set_project_root(tmp_path)
        assert self.service.load_config("missing") == {}

    def test_get_config_value(self, tmp_path: Path) -> None:
        config_dir = tmp_path / "bin" / "config"
        config_dir.mkdir(parents=True, exist_ok=True)

        config_file = config_dir / "theme.config.yaml"
        with open(config_file, "w", encoding="utf-8") as file:
            yaml.safe_dump(
                {"palette": {"dark": {"$_main_surface": "rgb(0, 0, 0)"}}}, file
            )

        self.service.set_project_root(tmp_path)
        value = self.service.get_config_value(
            "theme", "palette.dark.$_main_surface", "fallback"
        )
        assert value == "rgb(0, 0, 0)"

    def test_save_config_writes_config_yaml(self, tmp_path: Path) -> None:
        self.service.set_project_root(tmp_path)

        data = {"app": {"name": "Updated App", "theme": "light"}}
        result = self.service.save_config("app", data)

        assert result is True
        saved_file = tmp_path / "bin" / "config" / "app.config.yaml"
        assert saved_file.exists()
        loaded = yaml.safe_load(saved_file.read_text(encoding="utf-8"))
        assert loaded["app"]["theme"] == "light"

    def test_get_package_resource_returns_existing_path(self) -> None:
        resource = get_package_resource("resources/config/app.config.yaml")
        assert isinstance(resource, Path)
        assert resource.exists()
        assert resource.name == "app.config.yaml"



# ///////////////////////////////////////////////////////////////
# DOMAIN.PORTS.CONFIG_SERVICE - Config service port
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Protocol definitions for configuration services."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
from pathlib import Path
from typing import Any, Protocol


# ///////////////////////////////////////////////////////////////
# PROTOCOLS
# ///////////////////////////////////////////////////////////////
class ConfigServiceProtocol(Protocol):
    """Technical contract for configuration services."""

    def set_project_root(self, project_root: Path | str) -> None:
        """Set the active project root directory."""

    def load_config(
        self, config_name: str, force_reload: bool = False
    ) -> dict[str, Any]:
        """Load a named configuration."""

    def get_config_value(
        self, config_name: str, key_path: str, default: Any = None
    ) -> Any:
        """Read a specific value from a configuration."""

    def save_config(self, config_name: str, config_data: dict[str, Any]) -> bool:
        """Persist a named configuration."""



# ///////////////////////////////////////////////////////////////
# WIDGETS.CORE.SETTINGS_PANEL - Settings panel widget
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""SettingsPanel widget with scrollable settings and YAML persistence."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
import contextlib
from typing import Any

# Third-party imports
from PySide6.QtCore import QSize, Qt, Signal
from PySide6.QtWidgets import (
    QFrame,
    QLabel,
    QScrollArea,
    QSizePolicy,
    QVBoxLayout,
    QWidget,
)

# Local imports
from ...services.settings import get_settings_service
from ...services.translation import get_translation_service
from ...services.ui import Fonts
from ...utils.diagnostics import warn_tech


# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////
class SettingsPanel(QFrame):
    """
    This class is used to create a settings panel.
    It contains a top border, a content settings frame and a theme settings container.
    The settings panel is used to display the settings.
    """

    # Signal emitted when a setting changes
    settingChanged = Signal(str, object)  # key, value
    # Signal emitted when language changes
    languageChanged = Signal()

    # ///////////////////////////////////////////////////////////////

    def _settings_storage_prefix(self) -> list[str]:
        """Return config path prefix used to persist settings values."""
        try:
            from ...services.config import get_config_service

            app_config = get_config_service().load_config("app")
            app_section = app_config.get("app", {})
            root = str(app_section.get("settings_storage_root", "app.settings_panel"))
            parts = [part for part in root.split(".") if part]
            if len(parts) >= 2:
                return parts
        except Exception as e:
            warn_tech(
                code="widgets.settings_panel.storage_prefix_resolution_failed",
                message="Could not resolve settings storage prefix from app config",
                error=e,
            )
        return ["app", "settings_panel"]

    def __init__(
        self,
        parent: QWidget | None = None,
        width: int = 240,
        load_from_yaml: bool = True,
    ) -> None:
        super().__init__(parent)
        self._widgets: list[QWidget] = []
        self._settings: dict[str, Any] = {}

        # ///////////////////////////////////////////////////////////////
        # Store configuration
        self._width = width

        self.setObjectName("settingsPanel")
        self.setMinimumSize(QSize(0, 0))
        self.setMaximumSize(QSize(0, 16777215))
        self.setFrameShape(QFrame.Shape.NoFrame)
        self.setFrameShadow(QFrame.Shadow.Raised)
        # //////
        self.VL_settingsPanel = QVBoxLayout(self)
        self.VL_settingsPanel.setSpacing(0)
        self.VL_settingsPanel.setObjectName("VL_settingsPanel")
        self.VL_settingsPanel.setContentsMargins(0, 0, 0, 0)

        # ///////////////////////////////////////////////////////////////

        self.settingsTopBorder = QFrame(self)
        self.settingsTopBorder.setObjectName("settingsTopBorder")
        self.settingsTopBorder.setMaximumSize(QSize(16777215, 3))
        self.settingsTopBorder.setFrameShape(QFrame.Shape.NoFrame)
        self.settingsTopBorder.setFrameShadow(QFrame.Shadow.Raised)
        #
        self.VL_settingsPanel.addWidget(self.settingsTopBorder)

        # ///////////////////////////////////////////////////////////////

        # Create QScrollArea for settings
        self.settingsScrollArea = QScrollArea(self)
        self.settingsScrollArea.setObjectName("settingsScrollArea")
        self.settingsScrollArea.setWidgetResizable(True)
        self.settingsScrollArea.setHorizontalScrollBarPolicy(
            Qt.ScrollBarPolicy.ScrollBarAlwaysOff
        )
        self.settingsScrollArea.setVerticalScrollBarPolicy(
            Qt.ScrollBarPolicy.ScrollBarAsNeeded
        )
        self.settingsScrollArea.setFrameShape(QFrame.Shape.NoFrame)
        self.settingsScrollArea.setFrameShadow(QFrame.Shadow.Raised)
        #
        self.VL_settingsPanel.addWidget(self.settingsScrollArea)

        # ///////////////////////////////////////////////////////////////

        # Container widget for all settings
        self.contentSettings = QFrame()
        self.contentSettings.setObjectName("contentSettings")
        self.contentSettings.setFrameShape(QFrame.Shape.NoFrame)
        self.contentSettings.setFrameShadow(QFrame.Shadow.Raised)
        self.contentSettings.setSizePolicy(
            QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Preferred
        )
        #
        self.settingsScrollArea.setWidget(self.contentSettings)
        # //////
        self.VL_contentSettings = QVBoxLayout(self.contentSettings)
        self.VL_contentSettings.setObjectName("VL_contentSettings")
        self.VL_contentSettings.setSpacing(0)
        self.VL_contentSettings.setContentsMargins(0, 0, 0, 0)
        self.VL_contentSettings.setAlignment(Qt.AlignmentFlag.AlignTop)

        # ///////////////////////////////////////////////////////////////

        self.themeSettingsContainer = QFrame(self.contentSettings)
        self.themeSettingsContainer.setObjectName("themeSettingsContainer")
        self.themeSettingsContainer.setFrameShape(QFrame.Shape.NoFrame)
        self.themeSettingsContainer.setFrameShadow(QFrame.Shadow.Raised)
        #
        self.VL_contentSettings.addWidget(
            self.themeSettingsContainer, 0, Qt.AlignmentFlag.AlignTop
        )
        # //////
        self.VL_themeSettingsContainer = QVBoxLayout(self.themeSettingsContainer)
        self.VL_themeSettingsContainer.setSpacing(8)
        self.VL_themeSettingsContainer.setObjectName("VL_themeSettingsContainer")
        self.VL_themeSettingsContainer.setContentsMargins(10, 10, 10, 10)

        # ///////////////////////////////////////////////////////////////

        self.themeLabel = QLabel("Active Theme", self.themeSettingsContainer)
        self.themeLabel.setObjectName("themeLabel")
        if Fonts.SEGOE_UI_10_SB is not None:
            self.themeLabel.setFont(Fonts.SEGOE_UI_10_SB)
        self.themeLabel.setAlignment(
            Qt.AlignmentFlag.AlignLeading
            | Qt.AlignmentFlag.AlignLeft
            | Qt.AlignmentFlag.AlignVCenter
        )
        #
        self.VL_themeSettingsContainer.addWidget(self.themeLabel)

        # ///////////////////////////////////////////////////////////////

        # Create theme selector if OptionSelector is available
        try:
            from ezqt_widgets import OptionSelector

            # Create theme selector with correct signature
            # OptionSelector expects: items, default_id, min_width, min_height, orientation, animation_duration, parent
            self.themeToggleButton = OptionSelector(
                items=["Light", "Dark"],  # List of options
                default_id=1,  # 0 = Light, 1 = Dark (Dark by default)
                min_width=None,
                min_height=None,
                orientation="horizontal",
                animation_duration=get_settings_service().gui.TIME_ANIMATION,
                parent=self.themeSettingsContainer,
            )
            self.themeToggleButton.setObjectName("themeToggleButton")
            self.themeToggleButton.setSizePolicy(
                QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed
            )
            self.themeToggleButton.setFixedHeight(40)
            self._widgets.append(self.themeToggleButton)

            # Connect theme selector change signal
            self._connect_theme_selector_signals()

            # Add to layout
            self.VL_themeSettingsContainer.addWidget(self.themeToggleButton)

        except ImportError:
            warn_tech(
                code="widgets.settings_panel.option_selector_unavailable",
                message="OptionSelector not available, theme toggle not created",
            )

        # ///////////////////////////////////////////////////////////////
        # Automatic loading from YAML if requested
        if load_from_yaml:
            self.load_settings_from_yaml()

        # Connect setting changes
        self.settingChanged.connect(self._on_setting_changed)

    # ///////////////////////////////////////////////////////////////

    def load_settings_from_yaml(self) -> None:
        """Load settings from YAML file."""
        try:
            from pathlib import Path

            import yaml

            # Try to load the full app.config.yaml file directly
            # Try multiple possible paths
            possible_paths = [
                Path.cwd() / "bin" / "config" / "app.config.yaml",  # User project
                Path(__file__).parent.parent.parent
                / "resources"
                / "config"
                / "app.config.yaml",  # Package
            ]

            app_config = None
            for path in possible_paths:
                if path.exists():
                    with open(path, encoding="utf-8") as f:
                        app_config = yaml.safe_load(f)
                    break

            if app_config is None:
                warn_tech(
                    code="widgets.settings_panel.app_config_yaml_not_found",
                    message="Could not find app.config.yaml file",
                )
                return

            # Extract settings_panel section
            settings_config = app_config.get("settings_panel", {})

            # Create widgets for each setting
            for key, config in settings_config.items():
                # Exclude theme as it's already manually managed by OptionSelector
                if key == "theme":
                    continue

                if config.get("enabled", True):  # Check if setting is enabled
                    widget = self.add_setting_from_config(key, config)

                    # Use default value from config (which may have been updated)
                    default_value = config.get("default")
                    if default_value is not None and hasattr(widget, "set_value"):
                        widget.set_value(default_value)  # type: ignore[union-attr]

        except Exception as e:
            warn_tech(
                code="widgets.settings_panel.load_yaml_failed",
                message="Error loading settings from YAML",
                error=e,
            )

    def add_setting_from_config(self, key: str, config: dict) -> QWidget:
        """Add a setting based on its YAML configuration."""
        setting_type = config.get("type", "text")
        label = config.get("label", key)
        description = config.get("description", "")
        default_value = config.get("default")

        # Create container for this setting (like themeSettingsContainer)
        setting_container = QFrame(self.contentSettings)
        setting_container.setObjectName(f"settingContainer_{key}")
        setting_container.setFrameShape(QFrame.Shape.NoFrame)
        setting_container.setFrameShadow(QFrame.Shadow.Raised)

        # Container layout with margins
        container_layout = QVBoxLayout(setting_container)
        container_layout.setSpacing(8)
        container_layout.setObjectName(f"VL_settingContainer_{key}")
        container_layout.setContentsMargins(10, 10, 10, 10)

        # Create widget based on type
        if setting_type == "toggle":
            widget = self._create_toggle_widget(
                label,
                description,
                bool(default_value) if default_value is not None else False,
                key,
            )
        elif setting_type == "select":
            options = config.get("options", [])
            widget = self._create_select_widget(
                label,
                description,
                options,
                str(default_value) if default_value is not None else "",
                key,
            )
        elif setting_type == "slider":
            min_val = config.get("min", 0)
            max_val = config.get("max", 100)
            unit = config.get("unit", "")
            widget = self._create_slider_widget(
                label,
                description,
                min_val,
                max_val,
                int(default_value) if default_value is not None else 0,
                unit,
                key,
            )
        elif setting_type == "checkbox":
            widget = self._create_checkbox_widget(
                label,
                description,
                bool(default_value) if default_value is not None else False,
                key,
            )
        else:  # text by default
            widget = self._create_text_widget(
                label,
                description,
                str(default_value) if default_value is not None else "",
                key,
            )

        # Add widget to container
        container_layout.addWidget(widget)

        # Add container to main layout
        self.VL_contentSettings.addWidget(setting_container)

        # Store reference
        self._settings[key] = widget

        return widget

    def _create_toggle_widget(
        self, label: str, description: str, default: bool, key: str | None = None
    ) -> QWidget:
        """Create a toggle widget with label and description."""
        from ...widgets.extended.setting_widgets import SettingToggle

        widget = SettingToggle(label, description, default)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    def _create_select_widget(
        self,
        label: str,
        description: str,
        options: list[str],
        default: str,
        key: str | None = None,
    ) -> QWidget:
        """Create a select widget with label and description."""
        from ...widgets.extended.setting_widgets import SettingSelect

        widget = SettingSelect(label, description, options, default)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    def _create_slider_widget(
        self,
        label: str,
        description: str,
        min_val: int,
        max_val: int,
        default: int,
        unit: str,
        key: str | None = None,
    ) -> QWidget:
        """Create a slider widget with label and description."""
        from ...widgets.extended.setting_widgets import SettingSlider

        widget = SettingSlider(label, description, min_val, max_val, default, unit)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    def _create_checkbox_widget(
        self, label: str, description: str, default: bool, key: str | None = None
    ) -> QWidget:
        """Create a checkbox widget with label and description."""
        from ...widgets.extended.setting_widgets import SettingCheckbox

        widget = SettingCheckbox(label, description, default)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    def _create_text_widget(
        self, label: str, description: str, default: str, key: str | None = None
    ) -> QWidget:
        """Create a text widget with label and description."""
        from ...widgets.extended.setting_widgets import SettingText

        widget = SettingText(label, description, default)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    # ///////////////////////////////////////////////////////////////
    # Simplified methods for manual setting addition

    def add_toggle_setting(
        self,
        key: str,
        label: str,
        default: bool = False,
        description: str = "",
        enabled: bool = True,  # noqa: ARG002
    ):
        """Add a toggle setting."""
        from ...widgets.extended.setting_widgets import SettingToggle

        widget = SettingToggle(label, description, default)
        widget._key = key  # Set the key
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def add_select_setting(
        self,
        key: str,
        label: str,
        options: list[str],
        default: str | None = None,
        description: str = "",
        enabled: bool = True,  # noqa: ARG002
    ):
        """Add a selection setting."""
        from ...widgets.extended.setting_widgets import SettingSelect

        widget = SettingSelect(label, description, options, default)
        widget._key = key  # Set the key
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def add_slider_setting(
        self,
        key: str,
        label: str,
        min_val: int,
        max_val: int,
        default: int,
        unit: str = "",
        description: str = "",
        enabled: bool = True,  # noqa: ARG002
    ):
        """Add a slider setting."""
        from ...widgets.extended.setting_widgets import SettingSlider

        widget = SettingSlider(label, description, min_val, max_val, default, unit)
        widget._key = key  # Set the key
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def add_text_setting(
        self,
        key: str,
        label: str,
        default: str = "",
        description: str = "",
        enabled: bool = True,  # noqa: ARG002
    ):
        """Add a text setting."""
        from ...widgets.extended.setting_widgets import SettingText

        widget = SettingText(label, description, default)
        widget._key = key  # Set the key
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def add_checkbox_setting(
        self,
        key: str,
        label: str,
        default: bool = False,
        description: str = "",
        enabled: bool = True,  # noqa: ARG002
    ):
        """Add a checkbox setting."""
        from ...widgets.extended.setting_widgets import SettingCheckbox

        widget = SettingCheckbox(label, description, default)
        widget._key = key  # Set the key
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def _on_setting_changed(self, key: str, value):
        """Called when a setting changes."""
        # Protection against recursion
        if not hasattr(self, "_processing_setting_change"):
            self._processing_setting_change = False

        if self._processing_setting_change:
            return  # Avoid recursion

        self._processing_setting_change = True

        try:
            # Save to YAML
            try:
                # Direct import to avoid circular import
                from ...services.application.app_service import AppService

                # Save directly to settings_panel[key].default
                AppService.write_yaml_config(
                    [*self._settings_storage_prefix(), key, "default"], value
                )
            except Exception as e:
                warn_tech(
                    code="widgets.settings_panel.save_setting_failed",
                    message=f"Could not save setting '{key}' to YAML",
                    error=e,
                )

            # Special handling for language changes
            if key == "language":
                try:
                    translation_service = get_translation_service()
                    # Check if language really changes
                    current_lang = translation_service.get_current_language_name()
                    if current_lang != str(value):
                        translation_service.change_language(str(value))
                        # Emit language change signal
                        self.languageChanged.emit()
                except Exception as e:
                    warn_tech(
                        code="widgets.settings_panel.change_language_failed",
                        message="Could not change language",
                        error=e,
                    )

            # Emit signal for application
            self.settingChanged.emit(key, value)
        finally:
            self._processing_setting_change = False

    # ///////////////////////////////////////////////////////////////
    # Utility methods

    def get_setting_value(self, key: str) -> Any:
        """Get the value of a setting."""
        if key in self._settings:
            return self._settings[key].get_value()
        return None

    def set_setting_value(self, key: str, value: Any) -> None:
        """Set the value of a setting."""
        if key in self._settings:
            self._settings[key].set_value(value)

    def get_all_settings(self) -> dict[str, Any]:
        """Get all settings and their values."""
        return {key: widget.get_value() for key, widget in self._settings.items()}

    def save_all_settings_to_yaml(self) -> None:
        """Save all settings to YAML file."""
        # Direct import to avoid circular import
        from ...services.application.app_service import AppService

        for key, widget in self._settings.items():
            try:
                AppService.write_yaml_config(
                    [*self._settings_storage_prefix(), key, "default"],
                    widget.get_value(),
                )
            except Exception as e:
                warn_tech(
                    code="widgets.settings_panel.save_all_settings_failed",
                    message=f"Could not save setting '{key}' to YAML",
                    error=e,
                )

    # ///////////////////////////////////////////////////////////////
    # Panel management methods

    def get_width(self) -> int:
        """Get panel width."""
        return self._width

    def set_width(self, width: int) -> None:
        """Set panel width."""
        self._width = width

    def get_theme_toggle_button(self):
        """Get the theme toggle button if available."""
        if hasattr(self, "themeToggleButton"):
            return self.themeToggleButton
        return None

    def update_all_theme_icons(self) -> None:
        """Update theme icons for all widgets that support it."""
        for widget in self._widgets:
            updater = getattr(widget, "update_theme_icon", None)
            if callable(updater):
                updater()

        # Force refresh of settings panel style
        self.style().unpolish(self)
        self.style().polish(self)

        # Also refresh all child widgets
        for child in self.findChildren(QWidget):
            child.style().unpolish(child)
            child.style().polish(child)

    def _connect_theme_selector_signals(self) -> None:
        """Connect theme selector signals."""
        with contextlib.suppress(Exception):
            if hasattr(self, "themeToggleButton"):
                # Connect OptionSelector valueChanged signal
                theme_button = self.themeToggleButton

                if hasattr(theme_button, "valueChanged"):
                    theme_button.valueChanged.connect(self._on_theme_selector_changed)
                elif hasattr(theme_button, "clicked"):
                    theme_button.clicked.connect(self._on_theme_selector_clicked)

    def _on_theme_selector_changed(self, value):
        """Called when theme selector value changes."""
        try:
            # OptionSelector.value already returns "Light" or "Dark"
            english_value = value.lower()

            # Save English value to YAML
            from ...services.application.app_service import AppService

            AppService.write_yaml_config(
                [*self._settings_storage_prefix(), "theme", "default"],
                english_value,
            )

            # Emit signal with English value
            self.settingChanged.emit("theme", english_value)

        except Exception as e:
            warn_tech(
                code="widgets.settings_panel.theme_selector_change_failed",
                message="Could not handle theme selector change",
                error=e,
            )

    def _on_theme_selector_clicked(self):
        """Called when theme selector is clicked."""
        try:
            if hasattr(self, "themeToggleButton"):
                # Get text value directly
                current_value = self.themeToggleButton.value.lower()

                # Save English value to YAML
                from ...services.application.app_service import AppService

                AppService.write_yaml_config(
                    [*self._settings_storage_prefix(), "theme", "default"],
                    current_value,
                )

                # Emit signal with English value
                self.settingChanged.emit("theme", current_value)

        except Exception as e:
            warn_tech(
                code="widgets.settings_panel.theme_selector_click_failed",
                message="Could not handle theme selector click",
                error=e,
            )

    def update_theme_selector_items(self) -> None:
        """Update theme selector items with translations."""
        with contextlib.suppress(Exception):
            if hasattr(self, "themeToggleButton"):
                from ...services.translation import tr

                # Translate items for display
                translated_items = [tr("Light"), tr("Dark")]

                # Save current value (ID)
                theme_button = self.themeToggleButton
                current_id = (
                    theme_button.value_id if hasattr(theme_button, "value_id") else 0
                )

                # Update widget texts directly
                if hasattr(theme_button, "_options"):
                    for i, (_option_id, option_widget) in enumerate(
                        theme_button._options.items()
                    ):
                        if i < len(translated_items):
                            if hasattr(option_widget, "label"):
                                option_widget.label.setText(
                                    translated_items[i].capitalize()
                                )
                            elif hasattr(option_widget, "setText"):
                                option_widget.setText(translated_items[i].capitalize())

                # Reapply current ID to maintain selection
                if hasattr(theme_button, "value_id") and hasattr(
                    theme_button, "_value_id"
                ):
                    theme_button._value_id = current_id
                    # Force selector movement
                    if current_id in theme_button._options:
                        theme_button.move_selector(theme_button._options[current_id])

    def add_setting_widget(self, widget: QWidget) -> None:
        """Add a new setting widget to the settings panel."""
        # Create container for setting (like themeSettingsContainer)
        setting_container = QFrame(self.contentSettings)
        setting_container.setObjectName(f"settingContainer_{widget.objectName()}")
        setting_container.setFrameShape(QFrame.Shape.NoFrame)
        setting_container.setFrameShadow(QFrame.Shadow.Raised)

        # Container layout with margins (like VL_themeSettingsContainer)
        container_layout = QVBoxLayout(setting_container)
        container_layout.setSpacing(8)
        container_layout.setContentsMargins(10, 10, 10, 10)

        # Add widget to container
        container_layout.addWidget(widget)

        # Add container to main layout
        self.VL_contentSettings.addWidget(setting_container)
        self._widgets.append(widget)

    def add_setting_section(self, title: str = "") -> QFrame:
        """Add a new settings section with optional title."""
        section = QFrame(self.contentSettings)
        section.setObjectName(f"settingsSection_{title.replace(' ', '_')}")
        section.setFrameShape(QFrame.Shape.NoFrame)
        section.setFrameShadow(QFrame.Shadow.Raised)

        section_layout = QVBoxLayout(section)
        section_layout.setSpacing(8)
        section_layout.setContentsMargins(10, 10, 10, 10)

        if title:
            title_label = QLabel(title, section)
            if Fonts.SEGOE_UI_10_REG is not None:
                title_label.setFont(Fonts.SEGOE_UI_10_REG)
            title_label.setAlignment(
                Qt.AlignmentFlag.AlignLeading
                | Qt.AlignmentFlag.AlignLeft
                | Qt.AlignmentFlag.AlignVCenter
            )
            section_layout.addWidget(title_label)

        self.VL_contentSettings.addWidget(section)
        return section

    def scroll_to_top(self) -> None:
        """Scroll to top of settings panel."""
        if hasattr(self, "settingsScrollArea"):
            self.settingsScrollArea.verticalScrollBar().setValue(0)

    def scroll_to_bottom(self) -> None:
        """Scroll to bottom of settings panel."""
        if hasattr(self, "settingsScrollArea"):
            scrollbar = self.settingsScrollArea.verticalScrollBar()
            scrollbar.setValue(scrollbar.maximum())

    def scroll_to_widget(self, widget: QWidget) -> None:
        """
        Scroll to a specific widget in the settings panel.

        Args:
            widget: The widget to scroll to
        """
        if hasattr(self, "settingsScrollArea") and widget:
            # Calculate widget position in scroll area
            widget_pos = widget.mapTo(self.contentSettings, widget.rect().topLeft())
            self.settingsScrollArea.verticalScrollBar().setValue(widget_pos.y())


# ///////////////////////////////////////////////////////////////
# PUBLIC API
# ///////////////////////////////////////////////////////////////
__all__ = ["SettingsPanel"]



"""
Theme service implementation for UI styling.
"""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
from pathlib import Path
from typing import Any

# Local imports
from ...utils.runtime_paths import APP_PATH
from ..config import get_config_service
from ..settings import get_settings_service


# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////
class ThemeService:
    """Service responsible for loading and applying QSS themes."""

    @staticmethod
    def apply_theme(window: Any, custom_theme_file: str | None = None) -> None:
        """Load and apply either a custom QSS file or the default package theme."""
        settings_service = get_settings_service()
        config_service = get_config_service()

        theme_name = settings_service.gui.THEME
        theme_config = config_service.load_config("theme")
        colors = theme_config.get("palette", {}).get(theme_name, {})

        main_style = ThemeService._load_theme_content(custom_theme_file)

        for key, color in colors.items():
            main_style = main_style.replace(key, color)

        window.ui.styleSheet.setStyleSheet(f"{main_style}\n")

    @staticmethod
    def _load_theme_content(custom_theme_file: str | None) -> str:
        """Resolve and read theme stylesheet content from local or package paths."""
        if custom_theme_file:
            custom_qss = APP_PATH / "bin" / "themes" / custom_theme_file
            if custom_qss.exists():
                return custom_qss.read_text(encoding="utf-8")
            raise FileNotFoundError(f"Custom theme file not found: {custom_qss}")

        local_qss = APP_PATH / "bin" / "themes" / "main_theme.qss"
        if local_qss.exists():
            return local_qss.read_text(encoding="utf-8")

        package_qss = (
            Path(__file__).resolve().parents[2]
            / "resources"
            / "themes"
            / "main_theme.qss"
        )
        if package_qss.exists():
            return package_qss.read_text(encoding="utf-8")

        raise FileNotFoundError(
            "Default theme file not found in local or package resources."
        )



# ///////////////////////////////////////////////////////////////
# SERVICES.APPLICATION.APP_SERVICE - Application orchestration service
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Central application service — orchestrates config, assets, resources and settings."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
from pathlib import Path
from typing import Any

# Local imports
from ..config.config_service import (
    get_config_service,
    get_package_resource,
    get_package_resource_content,
)
from .assets_service import AssetsService
from .resource_service import ResourceService
from .settings_loader import SettingsLoader


# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////
class AppService:
    """Central orchestration service for application lifecycle.

    Aggregates config, asset, resource and settings operations into a
    single cohesive snake_case API used by widgets, CLI and bootstrap.
    """

    # ------------------------------------------------------------------
    # Assets
    # ------------------------------------------------------------------

    @staticmethod
    def check_assets_requirements(
        base_path: Path | None = None,
        bin_path: Path | None = None,
        overwrite_policy: str = "ask",
    ) -> None:
        """Generate asset binaries, QRC and RC files at APP_PATH."""
        AssetsService.check_assets_requirements(
            base_path=base_path,
            bin_path=bin_path,
            overwrite_policy=overwrite_policy,
        )

    @staticmethod
    def make_required_files(
        mk_theme: bool = True,
        mk_config: bool = True,
        mk_translations: bool = True,
        base_path: Path | None = None,
        bin_path: Path | None = None,
        overwrite_policy: str = "ask",
    ) -> None:
        """Copy YAML, QSS theme and translation files into ``cwd/bin/``.

        Parameters
        ----------
        mk_theme:
            When ``True`` (default) also copies the QSS theme file.
        """
        AssetsService.make_required_files(
            mk_theme=mk_theme,
            mk_config=mk_config,
            mk_translations=mk_translations,
            base_path=base_path,
            bin_path=bin_path,
            overwrite_policy=overwrite_policy,
        )

    # ------------------------------------------------------------------
    # Configuration
    # ------------------------------------------------------------------

    @staticmethod
    def set_project_root(project_root: Path) -> None:
        """Set the project root directory used by the config service."""
        get_config_service().set_project_root(project_root)

    @staticmethod
    def load_config(config_name: str) -> dict[str, Any]:
        """Load a named configuration from its YAML file.

        Parameters
        ----------
        config_name:
            Logical name of the configuration (e.g. ``"app"``).
        """
        return get_config_service().load_config(config_name)

    @staticmethod
    def get_config_value(config_name: str, key_path: str, default: Any = None) -> Any:
        """Get a value from a named configuration using dot-separated path.

        Parameters
        ----------
        config_name:
            Logical name of the configuration.
        key_path:
            Dot-separated key path, e.g. ``"app.name"``.
        default:
            Value returned when the path is absent.
        """
        return get_config_service().get_config_value(config_name, key_path, default)

    @staticmethod
    def save_config(config_name: str, data: dict[str, Any]) -> None:
        """Persist a named configuration to its YAML file.

        Parameters
        ----------
        config_name:
            Logical name of the configuration.
        data:
            Full configuration dict to write.
        """
        get_config_service().save_config(config_name, data)

    @staticmethod
    def write_yaml_config(keys: list[str], val: Any) -> None:
        """Write a single value into a YAML config using a key list.

        The first element of *keys* is the config name; remaining elements
        form the nested path to the value.

        Parameters
        ----------
        keys:
            List of keys where ``keys[0]`` is the config name and
            ``keys[1:]`` is the path inside that config.
        val:
            Value to assign at the leaf key.
        """
        if not keys:
            return

        config_name = keys[0]
        config_service = get_config_service()
        config = config_service.load_config(config_name)

        current = config
        for key in keys[1:-1]:
            if key not in current:
                current[key] = {}
            current = current[key]

        current[keys[-1]] = val
        config_service.save_config(config_name, config)

    @staticmethod
    def copy_package_configs_to_project() -> None:
        """Copy package default configs into the child project directory."""
        get_config_service().copy_package_configs_to_project()

    # ------------------------------------------------------------------
    # Package resources
    # ------------------------------------------------------------------

    @staticmethod
    def get_package_resource(resource_path: str) -> Path:
        """Return a :class:`~pathlib.Path` to an installed package resource.

        Parameters
        ----------
        resource_path:
            Relative path inside the package (e.g. ``"resources/themes/main_theme.qss"``).
        """
        return get_package_resource(resource_path)

    @staticmethod
    def get_package_resource_content(resource_path: str) -> str:
        """Return the text content of an installed package resource.

        Parameters
        ----------
        resource_path:
            Relative path inside the package.
        """
        return get_package_resource_content(resource_path)

    # ------------------------------------------------------------------
    # Resources (fonts)
    # ------------------------------------------------------------------

    @staticmethod
    def load_fonts_resources(app: bool = False) -> None:
        """Load ``.ttf`` font files into Qt's font database.

        Parameters
        ----------
        app:
            When ``False`` loads from the installed package fonts then
            recurses with ``True`` to also load from ``bin/fonts/``.
        """
        ResourceService.load_fonts_resources(app)

    # ------------------------------------------------------------------
    # Settings
    # ------------------------------------------------------------------

    @staticmethod
    def load_app_settings() -> dict[str, Any]:
        """Load application settings from the project YAML file."""
        return SettingsLoader.load_app_settings()



# ///////////////////////////////////////////////////////////////
# SERVICES.BOOTSTRAP.INITIALIZER - Main application initializer
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""High-level initializer that coordinates the complete EzQt_App boot sequence."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
from pathlib import Path

# Local imports
from ...domain.results import InitResult
from ..application.app_service import AppService
from ..application.file_service import FileService
from .contracts.options import InitOptions, OverwritePolicy
from .init_service import InitService
from .sequence import InitializationSequence
from .startup_config import StartupConfig


# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////
class Initializer:
    """Coordinates the complete EzQt_App initialization process.

    Orchestrates:
    - Startup configuration (encoding, locale, env)
    - Asset and file generation via :class:`FileService`
    - Application requirements via :class:`AppService`
    - Idempotent guard (subsequent :meth:`initialize` calls are no-ops)
    """

    def __init__(self) -> None:
        self._startup_config = StartupConfig()
        self._file_service = FileService()
        self._sequence = InitializationSequence()
        self._init_service = InitService()
        self._initialized = False

    # ------------------------------------------------------------------
    # Main entry points
    # ------------------------------------------------------------------

    def initialize(
        self,
        _mk_theme: bool = True,
        verbose: bool = True,
        options: InitOptions | None = None,
    ) -> InitResult:
        """Run the complete boot sequence (idempotent).

        Parameters
        ----------
        mk_theme:
            Generate theme files (default: ``True``).
        verbose:
            Print progress information (default: ``True``).

        Returns
        -------
        dict
            Initialization summary (see :meth:`InitializationSequence.execute`).
        """
        resolved = options or InitOptions(
            mk_theme=_mk_theme,
            verbose=verbose,
            overwrite_policy=OverwritePolicy.ASK,
        )
        return self._init_service.run(resolved)

    def setup_project(self, base_path: str | None = None) -> bool:
        """Configure startup then scaffold a new project directory.

        Parameters
        ----------
        base_path:
            Root directory for the new project.  Defaults to ``cwd``.
        """
        self._startup_config.configure()
        return FileService(Path(base_path) if base_path else None).setup_project()

    # ------------------------------------------------------------------
    # Granular helpers
    # ------------------------------------------------------------------

    def configure_startup(self) -> None:
        """Run system-level startup configuration only."""
        self._startup_config.configure()

    def generate_assets(self) -> bool:
        """Generate all required asset files."""
        return self._file_service.generate_all_assets()

    def check_requirements(self) -> bool:
        """Return ``True`` if asset requirements are satisfied."""
        try:
            AppService.check_assets_requirements()
            return True
        except Exception:
            return False

    def make_required_files(self, mk_theme: bool = True) -> None:
        """Generate required config/resource files."""
        AppService.make_required_files(mk_theme)

    # ------------------------------------------------------------------
    # State
    # ------------------------------------------------------------------

    def is_initialized(self) -> bool:
        return self._initialized

    def reset(self) -> None:
        self._initialized = False
        self._init_service.reset()
        self._startup_config.reset()
        self._sequence.reset()

    # ------------------------------------------------------------------
    # Accessors (kept for compat)
    # ------------------------------------------------------------------

    def get_startup_config(self) -> StartupConfig:
        return self._startup_config

    def get_file_service(self) -> FileService:
        return self._file_service

    def get_sequence(self) -> InitializationSequence:
        return self._sequence



# ///////////////////////////////////////////////////////////////
# SHARED.RESOURCES - Shared resource definitions
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Shared static resource path definitions."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
import importlib.util
import sys
from pathlib import Path
from types import ModuleType
from typing import TypeVar, cast

# Local imports
from ...resources import base_resources_rc  # noqa: F401
from ...utils.runtime_paths import APP_PATH
from .icons import Icons as _PackageIcons
from .images import Images as _PackageImages

# Expose packaged defaults early so optional runtime modules importing
# ezqt_app.shared.resources can resolve these names during bootstrap.
Icons = _PackageIcons
Images = _PackageImages

T = TypeVar("T")


def _load_module_from_file(module_name: str, file_path: Path) -> ModuleType | None:
    """Import a Python module directly from file when it exists."""
    if not file_path.exists() or not file_path.is_file():
        return None

    spec = importlib.util.spec_from_file_location(module_name, file_path)
    if spec is None or spec.loader is None:
        return None

    module = importlib.util.module_from_spec(spec)
    try:
        spec.loader.exec_module(module)
    except Exception:
        return None

    sys.modules[module_name] = module
    return module


def _select_resource_class(
    module: ModuleType | None,
    class_names: tuple[str, ...],
    fallback: type[T],
) -> type[T]:
    """Return the first matching class from a module, otherwise fallback."""
    if module is None:
        return fallback

    for class_name in class_names:
        candidate = getattr(module, class_name, None)
        if isinstance(candidate, type):
            return cast(type[T], candidate)
    return fallback


def _load_runtime_resources() -> tuple[type[_PackageIcons], type[_PackageImages]]:
    """Load runtime-generated resources from project folders when available."""
    # Register user-generated Qt resources without replacing packaged fallback.
    _load_module_from_file(
        "ezqt_app._runtime_resources_rc", APP_PATH / "bin" / "resources_rc.py"
    )

    runtime_icons_module = _load_module_from_file(
        "ezqt_app._runtime_icons", APP_PATH / "modules" / "icons.py"
    )
    runtime_images_module = _load_module_from_file(
        "ezqt_app._runtime_images", APP_PATH / "modules" / "images.py"
    )
    runtime_app_module = _load_module_from_file(
        "ezqt_app._runtime_app_resources", APP_PATH / "modules" / "app_resources.py"
    )

    icons_cls = _select_resource_class(
        runtime_icons_module,
        ("AppIcons", "Icons"),
        _select_resource_class(
            runtime_app_module, ("AppIcons", "Icons"), _PackageIcons
        ),
    )
    images_cls = _select_resource_class(
        runtime_images_module,
        ("AppImages", "Images"),
        _select_resource_class(
            runtime_app_module, ("AppImages", "Images"), _PackageImages
        ),
    )
    return icons_cls, images_cls


_runtime_icons, _runtime_images = _load_runtime_resources()
Icons: type[_PackageIcons] = _runtime_icons
Images: type[_PackageImages] = _runtime_images

# ///////////////////////////////////////////////////////////////
# PUBLIC API
# ///////////////////////////////////////////////////////////////
__all__ = ["Icons", "Images"]



# ///////////////////////////////////////////////////////////////
# SERVICES.CONFIG.CONFIG_SERVICE - Config service implementation
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Configuration service — full implementation (absorbs ConfigManager logic)."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
import shutil
import sys
from pathlib import Path
from typing import Any

# Third-party imports
import yaml

# Local imports
from ...domain.ports.config_service import ConfigServiceProtocol
from ...utils.diagnostics import warn_user
from ...utils.printer import get_printer
from ...utils.runtime_paths import APP_PATH


# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////
class ConfigService(ConfigServiceProtocol):
    """Modular configuration service for EzQt_App."""

    def __init__(self):
        self._config_cache: dict[str, Any] = {}
        self._config_files: dict[str, Path] = {}
        self._project_root: Path | None = None

    # -----------------------------------------------------------
    # Port methods (ConfigServiceProtocol)
    # -----------------------------------------------------------

    def set_project_root(self, project_root: Path | str) -> None:
        """Set the active project root directory."""
        self._project_root = (
            project_root if isinstance(project_root, Path) else Path(project_root)
        )

    def load_config(
        self, config_name: str, force_reload: bool = False
    ) -> dict[str, Any]:
        """Load a named configuration from the first matching path.

        Parameters
        ----------
        config_name:
            Configuration file name (without extension, e.g. ``"app"``).
        force_reload:
            Bypass cache and reload from disk.

        Returns
        -------
        dict[str, Any]
            Loaded configuration data, or empty dict on failure.
        """
        if not force_reload and config_name in self._config_cache:
            return self._config_cache[config_name]

        config_file = self._resolve_config_file(config_name)

        if not config_file:
            warn_user(
                code="config.service.missing_file",
                user_message=f"No configuration file found for '{config_name}'",
                log_message=f"No configuration file found for '{config_name}'",
            )
            get_printer().verbose_msg(
                f"Searched paths: {self.get_config_paths(config_name)}"
            )
            return {}

        try:
            with open(config_file, encoding="utf-8") as f:
                config_data = yaml.safe_load(f)

            self._config_cache[config_name] = config_data
            self._config_files[config_name] = config_file

            get_printer().verbose_msg(
                f"Configuration '{config_name}' loaded from: {config_file}"
            )
            return config_data

        except Exception as e:
            get_printer().error(f"Error loading '{config_name}': {e}")
            return {}

    def get_config_value(
        self, config_name: str, key_path: str, default: Any = None
    ) -> Any:
        """Read a specific value from a configuration using dot-notation key path.

        Parameters
        ----------
        config_name:
            Configuration file name.
        key_path:
            Dot-separated path (e.g. ``"app.name"`` or ``"palette.dark"``).
        default:
            Value returned when key is absent.
        """
        config = self.load_config(config_name)
        keys = key_path.split(".")
        current = config

        for key in keys:
            if isinstance(current, dict) and key in current:
                current = current[key]
            else:
                return default

        return current

    def save_config(self, config_name: str, config_data: dict[str, Any]) -> bool:
        """Persist a named configuration to the project directory.

        Parameters
        ----------
        config_name:
            Configuration file name.
        config_data:
            Data to serialise as YAML.

        Returns
        -------
        bool
            ``True`` if the write succeeded.
        """
        if not self._project_root:
            get_printer().error("No project root defined")
            return False

        config_dir = self._project_root / "bin" / "config"
        config_dir.mkdir(parents=True, exist_ok=True)
        config_file = config_dir / f"{config_name}.config.yaml"

        try:
            with open(config_file, "w", encoding="utf-8") as f:
                yaml.dump(config_data, f, default_flow_style=False, allow_unicode=True)

            self._config_cache[config_name] = config_data
            self._config_files[config_name] = config_file

            get_printer().verbose_msg(
                f"Configuration '{config_name}' saved: {config_file}"
            )
            return True

        except Exception as e:
            get_printer().error(f"Error saving '{config_name}': {e}")
            return False

    # -----------------------------------------------------------
    # Implementation-specific methods
    # -----------------------------------------------------------

    def get_config_paths(self, config_name: str) -> list[Path]:
        """Return candidate paths for *config_name* in priority order.

        Parameters
        ----------
        config_name:
            Logical configuration name (e.g. ``"app"``, ``"languages"``,
            ``"theme"``). Files are resolved as ``<name>.config.yaml``.
        """
        config_file = f"{config_name}.config.yaml"
        paths: list[Path] = []

        # 1. Child project (bin/config/)
        if self._project_root:
            paths.append(self._project_root / "bin" / "config" / config_file)

        # 2. Current directory (bin/config/)
        paths.append(Path.cwd() / "bin" / "config" / config_file)

        # 3. Package resources — relative to cwd
        paths.append(Path.cwd() / "ezqt_app" / "resources" / "config" / config_file)

        # 4. Package resources — relative to APP_PATH
        paths.append(APP_PATH / "resources" / "config" / config_file)

        return paths

    def copy_package_configs_to_project(self) -> bool:
        """Copy package configuration files into the child project.

        Returns
        -------
        bool
            ``True`` if the operation succeeded.
        """
        if not self._project_root:
            get_printer().error("No project root defined")
            return False

        package_config_dir = self._find_package_config_dir()

        if not package_config_dir:
            get_printer().error(
                f"EzQt_App package not found. Tested paths: "
                f"{[str(p) for p in [Path.cwd(), APP_PATH]]}"
            )
            return False

        project_config_dir = self._project_root / "bin" / "config"
        project_config_dir.mkdir(parents=True, exist_ok=True)

        copied_files: list[str] = []

        try:
            for config_file in package_config_dir.glob("*.yaml"):
                target_file = project_config_dir / config_file.name

                if target_file.exists():
                    get_printer().verbose_msg(f"Existing file, ignored: {target_file}")
                    continue

                shutil.copy2(config_file, target_file)
                copied_files.append(config_file.name)
                get_printer().info(f"Configuration copied: {config_file.name}")

            if copied_files:
                get_printer().info(
                    f"✅ {len(copied_files)} configurations copied to project"
                )

            return True

        except Exception as e:
            get_printer().error(f"Error copying configurations: {e}")
            return False

    def clear_cache(self) -> None:
        """Invalidate the in-memory configuration cache."""
        self._config_cache.clear()
        self._config_files.clear()
        get_printer().verbose_msg("Configuration cache cleared")

    def get_loaded_configs(self) -> dict[str, Path]:
        """Return a snapshot of currently cached configuration paths."""
        return self._config_files.copy()

    # -----------------------------------------------------------
    # Private helpers
    # -----------------------------------------------------------

    def _resolve_config_file(self, config_name: str) -> Path | None:
        """Return the first existing path for *config_name*, or ``None``."""
        for path in self.get_config_paths(config_name):
            if path.exists():
                return path
        return None

    def _find_package_config_dir(self) -> Path | None:
        """Locate the ``resources/config`` directory inside the installed package."""
        package_root = _get_installed_package_root()
        if package_root is not None:
            installed_config_dir = package_root / "resources" / "config"
            if installed_config_dir.exists():
                return installed_config_dir

        # 1. Walk up from cwd looking for an ezqt_app package dir
        current = Path.cwd()
        while current.parent != current:
            candidate = current / "ezqt_app"
            if candidate.exists() and (candidate / "resources" / "config").exists():
                config_dir = candidate / "resources" / "config"
                get_printer().verbose_msg(f"EzQt_App package found: {candidate}")
                get_printer().verbose_msg(f"Configuration directory: {config_dir}")
                return config_dir
            current = current.parent

        # 2. Search sys.path entries
        for path in sys.path:
            candidate = Path(path) / "ezqt_app"
            if candidate.exists() and (candidate / "resources" / "config").exists():
                return candidate / "resources" / "config"

        # 3. Fallback — legacy known paths
        for fallback in [
            Path.cwd() / "ezqt_app" / "resources" / "config",
            APP_PATH / "resources" / "config",
            APP_PATH / "ezqt_app" / "resources" / "config",
        ]:
            if fallback.exists():
                get_printer().verbose_msg(
                    f"Configuration directory found (fallback): {fallback}"
                )
                return fallback

        return None


# ///////////////////////////////////////////////////////////////
# SINGLETONS
# ///////////////////////////////////////////////////////////////
_config_service: ConfigService | None = None


def _get_installed_package_root() -> Path | None:
    """Return installed ``ezqt_app`` package root when discoverable."""
    try:
        import importlib.util

        spec = importlib.util.find_spec("ezqt_app")
        if spec is not None:
            if spec.submodule_search_locations:
                location = next(iter(spec.submodule_search_locations), None)
                if location:
                    candidate = Path(location)
                    if candidate.exists():
                        return candidate
            if spec.origin:
                candidate = Path(spec.origin).resolve().parent
                if candidate.exists():
                    return candidate
    except Exception as e:
        get_printer().verbose_msg(
            f"Could not resolve installed package root for ezqt_app: {e}"
        )

    return None


def _resource_candidates(resource_path: str) -> list[Path]:
    """Build candidate paths for a package resource in priority order."""
    rel_path = Path(resource_path.replace("\\", "/"))
    candidates: list[Path] = []

    package_root = _get_installed_package_root()
    if package_root is not None:
        candidates.append(package_root / rel_path)

    candidates.extend(
        [
            APP_PATH / "ezqt_app" / rel_path,
            APP_PATH / rel_path,
            Path.cwd() / "ezqt_app" / rel_path,
        ]
    )
    return candidates


# ///////////////////////////////////////////////////////////////
# FUNCTIONS
# ///////////////////////////////////////////////////////////////
def get_config_service() -> ConfigService:
    """Return the singleton configuration service instance."""
    global _config_service
    if _config_service is None:
        _config_service = ConfigService()
    return _config_service


def get_package_resource(resource_path: str) -> Path:
    """Return the filesystem path of an installed package resource.

    Parameters
    ----------
    resource_path:
        Resource path relative to the ``ezqt_app`` package root.

    Returns
    -------
    Path
        Resolved path to the resource.
    """
    for candidate in _resource_candidates(resource_path):
        if candidate.exists():
            return candidate

    try:
        import pkg_resources  # type: ignore[import-untyped]

        pkg_candidate = Path(pkg_resources.resource_filename("ezqt_app", resource_path))
        if pkg_candidate.exists():
            return pkg_candidate
    except Exception as e:
        get_printer().verbose_msg(
            f"pkg_resources lookup failed for '{resource_path}': {e}"
        )

    package_root = _get_installed_package_root()
    if package_root is not None:
        return package_root / Path(resource_path.replace("\\", "/"))
    return APP_PATH / "ezqt_app" / Path(resource_path.replace("\\", "/"))


def get_package_resource_content(resource_path: str) -> str:
    """Return the decoded UTF-8 content of an installed package resource.

    Parameters
    ----------
    resource_path:
        Resource path relative to the ``ezqt_app`` package root.

    Returns
    -------
    str
        Resource file content.

    Raises
    ------
    FileNotFoundError
        If the resource cannot be located by any strategy.
    """
    try:
        import pkg_resources  # type: ignore[import-untyped]

        return pkg_resources.resource_string("ezqt_app", resource_path).decode("utf-8")
    except Exception:
        file_path = get_package_resource(resource_path)
        if file_path.exists():
            with open(file_path, encoding="utf-8") as f:
                return f.read()
        raise FileNotFoundError(f"Resource not found: {resource_path}") from None



# ///////////////////////////////////////////////////////////////
# DOMAIN.MODELS.SETTINGS - Settings state models
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Pure domain models for mutable application settings state."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
from dataclasses import dataclass, field


# ///////////////////////////////////////////////////////////////
# DATACLASSES
# ///////////////////////////////////////////////////////////////
@dataclass(slots=True)
class AppSettingsModel:
    """Mutable application settings values."""

    NAME: str = "MyApplication"
    DESCRIPTION: str = "MyDescription"
    ENABLE_CUSTOM_TITLE_BAR: bool = True
    APP_MIN_SIZE: tuple[int, int] = (940, 560)
    APP_WIDTH: int = 1280
    APP_HEIGHT: int = 720


@dataclass(slots=True)
class GuiSettingsModel:
    """Mutable GUI settings values."""

    THEME: str = "dark"
    MENU_PANEL_SHRINKED_WIDTH: int = 60
    MENU_PANEL_EXTENDED_WIDTH: int = 240
    SETTINGS_PANEL_WIDTH: int = 240
    TIME_ANIMATION: int = 400


@dataclass(slots=True)
class SettingsStateModel:
    """Mutable aggregate application settings state."""

    app: AppSettingsModel = field(default_factory=AppSettingsModel)
    gui: GuiSettingsModel = field(default_factory=GuiSettingsModel)



# ///////////////////////////////////////////////////////////////
# SERVICES.TRANSLATION.STRING_COLLECTOR - UI string collector
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Collects translatable strings from Qt widgets for translation workflows."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
import json
import re
from datetime import datetime
from pathlib import Path
from typing import Any

# Local imports
from ...utils.diagnostics import warn_tech, warn_user
from ...utils.printer import get_printer


# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////
class StringCollector:
    """String collector with language detection and task generation."""

    def __init__(self, user_dir: Path | None = None):
        if user_dir is None:
            user_dir = Path.home() / ".ezqt"

        self.user_dir = user_dir
        self.user_dir.mkdir(parents=True, exist_ok=True)

        self.translations_dir = self.user_dir / "translations"
        self.cache_dir = self.user_dir / "cache"
        self.logs_dir = self.user_dir / "logs"

        self.translations_dir.mkdir(exist_ok=True)
        self.cache_dir.mkdir(exist_ok=True)
        self.logs_dir.mkdir(exist_ok=True)

        self.pending_file = self.translations_dir / "pending_strings.txt"
        self.processed_file = self.translations_dir / "processed_strings.txt"
        self.language_detected_file = self.translations_dir / "language_detected.txt"
        self.translation_tasks_file = self.translations_dir / "translation_tasks.json"

        self._collected_strings: set[str] = set()
        self._new_strings: set[str] = set()
        self._language_detected_strings: list[tuple[str, str]] = []

    def collect_strings_from_widget(
        self, widget: Any, recursive: bool = True
    ) -> set[str]:
        collected: set[str] = set()

        def collect_recursive(w: Any) -> None:
            try:
                if hasattr(w, "text") and callable(getattr(w, "text", None)):
                    try:
                        text = w.text().strip()
                        if self._is_valid_string(text):
                            collected.add(text)
                    except Exception as e:
                        warn_tech(
                            code="translation.collector.read_text_failed",
                            message="Could not read widget text",
                            error=e,
                        )

                if hasattr(w, "toolTip") and callable(getattr(w, "toolTip", None)):
                    try:
                        tooltip = w.toolTip().strip()
                        if self._is_valid_string(tooltip):
                            collected.add(tooltip)
                    except Exception as e:
                        warn_tech(
                            code="translation.collector.read_tooltip_failed",
                            message="Could not read widget toolTip",
                            error=e,
                        )

                if hasattr(w, "placeholderText") and callable(
                    getattr(w, "placeholderText", None)
                ):
                    try:
                        placeholder = w.placeholderText().strip()
                        if self._is_valid_string(placeholder):
                            collected.add(placeholder)
                    except Exception as e:
                        warn_tech(
                            code="translation.collector.read_placeholder_failed",
                            message="Could not read widget placeholderText",
                            error=e,
                        )

                if hasattr(w, "windowTitle") and callable(
                    getattr(w, "windowTitle", None)
                ):
                    try:
                        title = w.windowTitle().strip()
                        if self._is_valid_string(title):
                            collected.add(title)
                    except Exception as e:
                        warn_tech(
                            code="translation.collector.read_window_title_failed",
                            message="Could not read widget windowTitle",
                            error=e,
                        )

                if recursive:
                    try:
                        for child in w.findChildren(type(w)):
                            collect_recursive(child)
                    except Exception as e:
                        warn_tech(
                            code="translation.collector.iter_children_failed",
                            message="Could not iterate widget children",
                            error=e,
                        )

            except Exception as e:
                warn_tech(
                    code="translation.collector.scan_widget_failed",
                    message=f"Error scanning widget {type(w)}",
                    error=e,
                )

        collect_recursive(widget)
        return collected

    def _is_valid_string(self, text: str) -> bool:
        if not text or len(text) < 2:
            return False
        if (
            text.startswith(("_", "menu_", "btn_", "setting"))
            or text.isdigit()
            or text in ("", " ", "\n", "\t")
        ):
            return False
        technical_patterns = [
            r"^[A-Z_]+$",
            r"^[a-z_]+$",
            r"^[a-z]+[A-Z][a-z]+$",
            r"^[A-Z][a-z]+[A-Z][a-z]+$",
        ]
        return not any(re.match(p, text) for p in technical_patterns)

    def _detect_language(self, text: str) -> str:
        try:
            from langdetect import (
                DetectorFactory,
                detect,
            )

            DetectorFactory.seed = 0
            return detect(text)
        except ImportError:
            return self._simple_language_detection(text)
        except Exception as e:
            warn_tech(
                code="translation.collector.language_detection_failed",
                message="Language detection error",
                error=e,
            )
            return "en"

    def _simple_language_detection(self, text: str) -> str:
        french_chars = "àáâãäåæçèéêëìíîïðñòóôõöøùúûüýþÿÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞŸ"
        german_chars = "äöüßÄÖÜ"
        spanish_chars = "ñáéíóúüÑÁÉÍÓÚÜ"
        if any(c in french_chars for c in text):
            return "fr"
        if any(c in german_chars for c in text):
            return "de"
        if any(c in spanish_chars for c in text):
            return "es"
        return "en"

    def save_pending_strings(self, strings: set[str]) -> None:
        try:
            sorted_strings = sorted(strings)
            with open(self.pending_file, "w", encoding="utf-8") as f:
                f.write(f"# Pending strings - {datetime.now().isoformat()}\n")
                f.write(f"# Total: {len(sorted_strings)} strings\n\n")
                for s in sorted_strings:
                    f.write(f"{s}\n")
            get_printer().info(f"✅ {len(strings)} pending strings saved")
        except Exception as e:
            warn_tech(
                code="translation.collector.save_pending_failed",
                message="Error saving strings",
                error=e,
            )

    def detect_languages_and_save(self, strings: set[str]) -> list[tuple[str, str]]:
        language_detected: list[tuple[str, str]] = []
        for text in strings:
            try:
                lang = self._detect_language(text)
                language_detected.append((lang, text))
            except Exception as e:
                warn_tech(
                    code="translation.collector.detect_languages_failed",
                    message="Language detection error",
                    error=e,
                )
                language_detected.append(("en", text))

        try:
            sorted_results = sorted(language_detected, key=lambda x: x[0])
            with open(self.language_detected_file, "w", encoding="utf-8") as f:
                f.write(
                    f"# Strings with detected language - {datetime.now().isoformat()}\n"
                )
                f.write("# Format: language_code|text\n\n")
                for lang, text in sorted_results:
                    f.write(f"{lang}|{text}\n")
            self._language_detected_strings = language_detected
        except Exception as e:
            warn_tech(
                code="translation.collector.save_language_results_failed",
                message="Error saving language detection results",
                error=e,
            )

        return language_detected

    def load_processed_strings(self) -> set[str]:
        processed: set[str] = set()
        try:
            if self.processed_file.exists():
                with open(self.processed_file, encoding="utf-8") as f:
                    for line in f:
                        line = line.strip()
                        if line and not line.startswith("#"):
                            processed.add(line)
                get_printer().info(f"✅ {len(processed)} processed strings loaded")
            else:
                get_printer().info("📝 No processed strings file found")
        except Exception as e:
            warn_tech(
                code="translation.collector.load_processed_failed",
                message="Error loading processed strings",
                error=e,
            )
        return processed

    def get_supported_languages(self) -> list[str]:
        return ["en", "fr", "de", "es", "it", "pt", "ru", "ja", "ko", "zh"]

    def generate_translation_tasks(
        self, language_detected: list[tuple[str, str]]
    ) -> dict[str, Any]:
        tasks: dict[str, Any] = {}
        supported = self.get_supported_languages()
        for source_lang, text in language_detected:
            tasks.setdefault(source_lang, {})
            for target_lang in (lang for lang in supported if lang != source_lang):
                tasks[source_lang].setdefault(target_lang, []).append(text)
        try:
            with open(self.translation_tasks_file, "w", encoding="utf-8") as f:
                json.dump(tasks, f, indent=2, ensure_ascii=False)
            get_printer().info(f"✅ {len(tasks)} translation tasks generated")
        except Exception as e:
            warn_tech(
                code="translation.collector.save_tasks_failed",
                message="Error saving tasks",
                error=e,
            )
        return tasks

    def get_new_strings(self) -> set[str]:
        return self._new_strings.copy()

    def collect_and_compare(
        self, widget: Any, recursive: bool = True
    ) -> dict[str, Any]:
        collected = self.collect_strings_from_widget(widget, recursive)
        self.save_pending_strings(collected)
        language_detected = self.detect_languages_and_save(collected)
        tasks = self.generate_translation_tasks(language_detected)
        processed = self.load_processed_strings()
        self._new_strings = collected - processed

        stats: dict[str, Any] = {
            "total_collected": len(collected),
            "total_processed": len(processed),
            "new_strings": len(self._new_strings),
            "languages_detected": len({lang for lang, _ in language_detected}),
            "translation_tasks": len(tasks),
        }
        get_printer().info("📊 Collection summary:")
        for key, val in stats.items():
            get_printer().info(f"  - {key}: {val}")
        return stats

    def mark_strings_as_processed(self, strings: set[str] | None = None) -> None:
        if strings is None:
            strings = self._new_strings
        if not strings:
            warn_user(
                code="translation.collector.mark_processed_empty",
                user_message="No strings to mark as processed",
            )
            return
        try:
            processed = self.load_processed_strings()
            processed.update(strings)
            sorted_strings = sorted(processed)
            with open(self.processed_file, "w", encoding="utf-8") as f:
                f.write(f"# Processed strings - {datetime.now().isoformat()}\n")
                f.write(f"# Total: {len(sorted_strings)} strings\n\n")
                for s in sorted_strings:
                    f.write(f"{s}\n")
            get_printer().info(f"✅ {len(strings)} strings marked as processed")
        except Exception as e:
            warn_user(
                code="translation.collector.mark_processed_failed",
                user_message="Error marking strings",
                log_message="Error marking strings",
                error=e,
            )

    def get_stats(self) -> dict[str, Any]:
        return {
            "collected_strings": len(self._collected_strings),
            "new_strings": len(self._new_strings),
            "language_detected": len(self._language_detected_strings),
            "pending_file": str(self.pending_file),
            "processed_file": str(self.processed_file),
            "language_detected_file": str(self.language_detected_file),
            "translation_tasks_file": str(self.translation_tasks_file),
        }

    def clear_cache(self) -> None:
        self._collected_strings.clear()
        self._new_strings.clear()
        self._language_detected_strings.clear()
        get_printer().info("Collector cache cleared")


# ///////////////////////////////////////////////////////////////
# SINGLETON
# ///////////////////////////////////////////////////////////////
_string_collector_instance: StringCollector | None = None


def get_string_collector(user_dir: Path | None = None) -> StringCollector:
    """Return the global StringCollector singleton."""
    global _string_collector_instance
    if _string_collector_instance is None:
        _string_collector_instance = StringCollector(user_dir)
    return _string_collector_instance



# ///////////////////////////////////////////////////////////////
# Test Runner Script
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""Test runner script for ezqt_app.

This script provides a convenient way to run tests with various options:
- Unit, integration, or robustness tests
- Coverage reports
- Parallel execution
- Custom markers
"""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Standard library imports
import argparse
import subprocess
import sys
from pathlib import Path

# ///////////////////////////////////////////////////////////////
# FUNCTIONS
# ///////////////////////////////////////////////////////////////


def run_command(cmd, description):
    """Execute a command and display the result."""
    print(f"\n{'=' * 60}")
    print(f"🚀 {description}")
    print(f"{'=' * 60}")

    try:
        # Stream stdout/stderr in real time instead of buffering everything.
        result = subprocess.run(cmd, shell=True, check=False)
        return result.returncode == 0
    except KeyboardInterrupt:
        print("\n⚠️  User interruption (Ctrl+C)")
        return False
    except Exception as e:
        print(f"❌ Error during execution: {e}")
        return False


def main():
    """Main function."""
    parser = argparse.ArgumentParser(description="Test runner for EzQt_App")
    parser.add_argument(
        "--type",
        choices=["unit", "integration", "robustness", "all"],
        default="unit",
        help="Type of tests to run",
    )
    parser.add_argument(
        "--coverage", action="store_true", help="Generate coverage report"
    )
    parser.add_argument("--verbose", action="store_true", help="Mode verbeux")
    parser.add_argument("--fast", action="store_true", help="Exclure les tests lents")
    parser.add_argument(
        "--parallel",
        action="store_true",
        help="Execute tests in parallel (pytest-xdist)",
    )
    parser.add_argument(
        "--marker",
        type=str,
        help="Execute only tests with this marker (e.g., wizard, config)",
    )
    args = parser.parse_args()

    # ////// CHECK THAT WE ARE IN THE RIGHT DIRECTORY
    if not Path("pyproject.toml").exists():
        print(
            "❌ Error: pyproject.toml not found. Run this script from the project root."
        )
        sys.exit(1)

    cmd_parts = [sys.executable, "-m", "pytest"]
    if args.verbose:
        cmd_parts.append("-v")
    if args.fast:
        cmd_parts.extend(["-m", "not slow"])
    if args.marker:
        cmd_parts.extend(["-m", args.marker])
    if args.parallel:
        cmd_parts.extend(["-n", "auto"])
    if args.type == "unit":
        cmd_parts.append("tests/unit/")
    elif args.type == "integration":
        cmd_parts.append("tests/integration/")
    elif args.type == "robustness":
        cmd_parts.append("tests/robustness/")
    else:
        cmd_parts.append("tests/")
    if args.coverage:
        cmd_parts.extend(
            ["--cov=ezqt_app", "--cov-report=term-missing", "--cov-report=html:htmlcov"]
        )

    cmd = " ".join(cmd_parts)

    # ////// RUN THE TESTS
    success = run_command(cmd, f"Running {args.type} tests")

    if success:
        print("\n✅ Tests executed successfully!")

        if args.coverage:
            print("\n📊 Coverage report generated in htmlcov/")
            print("   Open htmlcov/index.html in your browser")
    else:
        print("\n❌ Test failure")
        sys.exit(1)


if __name__ == "__main__":
    main()



# ///////////////////////////////////////////////////////////////
# SHARED.RESOURCES.ICONS - Icons registry
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""
Shared icon resource registry.
"""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# CLASSES
# ///////////////////////////////////////////////////////////////


class Icons:
    """
    Application icon management class.

    This class contains paths to all icons
    used in the application, organized by category.
    """

    # ////// CORE ICONS
    icon_close: str = ":/icons/icons/icon_close.png"
    icon_maximize: str = ":/icons/icons/icon_maximize.png"
    icon_menu: str = ":/icons/icons/icon_menu.png"
    icon_minimize: str = ":/icons/icons/icon_minimize.png"
    icon_restore: str = ":/icons/icons/icon_restore.png"
    icon_settings: str = ":/icons/icons/icon_settings.png"

    # ////// MATERIAL ICONS
    material_circled_C: str = ":/icons/icons/material-circled-C.png"
    material_circled_K: str = ":/icons/icons/material-circled-K.png"
    material_copy_24: str = ":/icons/icons/material-copy-24.png"
    material_electricity: str = ":/icons/icons/material-electricity.png"
    material_play_blue: str = ":/icons/icons/material-play-blue.png"
    material_stop_red: str = ":/icons/icons/material-stop-red.png"

    # ////// CIL ICONS - ARROWS
    cil_arrow_bottom_2: str = ":/icons/icons/cil-arrow-bottom-2.png"
    cil_arrow_bottom: str = ":/icons/icons/cil-arrow-bottom.png"
    cil_arrow_circle_bottom: str = ":/icons/icons/cil-arrow-circle-bottom.png"
    cil_arrow_circle_left: str = ":/icons/icons/cil-arrow-circle-left.png"
    cil_arrow_circle_right: str = ":/icons/icons/cil-arrow-circle-right.png"
    cil_arrow_circle_top: str = ":/icons/icons/cil-arrow-circle-top.png"
    cil_arrow_left: str = ":/icons/icons/cil-arrow-left.png"
    cil_arrow_right: str = ":/icons/icons/cil-arrow-right.png"
    cil_arrow_top: str = ":/icons/icons/cil-arrow-top.png"

    # ////// CIL ICONS - NAVIGATION
    cil_chevron_bottom: str = ":/icons/icons/cil-chevron-bottom.png"
    cil_chevron_circle_down_alt: str = ":/icons/icons/cil-chevron-circle-down-alt.png"
    cil_chevron_circle_left_alt: str = ":/icons/icons/cil-chevron-circle-left-alt.png"
    cil_chevron_circle_right_alt: str = ":/icons/icons/cil-chevron-circle-right-alt.png"
    cil_chevron_circle_up_alt: str = ":/icons/icons/cil-chevron-circle-up-alt.png"
    cil_chevron_double_down: str = ":/icons/icons/cil-chevron-double-down.png"
    cil_chevron_double_left: str = ":/icons/icons/cil-chevron-double-left.png"
    cil_chevron_double_right: str = ":/icons/icons/cil-chevron-double-right.png"
    cil_chevron_double_up_alt: str = ":/icons/icons/cil-chevron-double-up-alt.png"
    cil_chevron_double_up: str = ":/icons/icons/cil-chevron-double-up.png"
    cil_chevron_left: str = ":/icons/icons/cil-chevron-left.png"
    cil_chevron_right: str = ":/icons/icons/cil-chevron-right.png"
    cil_chevron_top: str = ":/icons/icons/cil-chevron-top.png"

    # ////// CIL ICONS - ACTIONS
    cil_action_redo: str = ":/icons/icons/cil-action-redo.png"
    cil_action_undo: str = ":/icons/icons/cil-action-undo.png"
    cil_check_alt: str = ":/icons/icons/cil-check-alt.png"
    cil_check_circle: str = ":/icons/icons/cil-check-circle.png"
    cil_check: str = ":/icons/icons/cil-check.png"
    cil_minus: str = ":/icons/icons/cil-minus.png"
    cil_plus: str = ":/icons/icons/cil-plus.png"
    cil_x_circle: str = ":/icons/icons/cil-x-circle.png"
    cil_x: str = ":/icons/icons/cil-x.png"

    # ////// CIL ICONS - ALIGNMENT
    cil_align_center: str = ":/icons/icons/cil-align-center.png"
    cil_align_left: str = ":/icons/icons/cil-align-left.png"
    cil_align_right: str = ":/icons/icons/cil-align-right.png"
    cil_justify_center: str = ":/icons/icons/cil-justify-center.png"
    cil_justify_left: str = ":/icons/icons/cil-justify-left.png"
    cil_justify_right: str = ":/icons/icons/cil-justify-right.png"

    # ////// CIL ICONS - COMMUNICATION
    cil_at: str = ":/icons/icons/cil-at.png"
    cil_chat_bubble: str = ":/icons/icons/cil-chat-bubble.png"
    cil_comment_bubble: str = ":/icons/icons/cil-comment-bubble.png"
    cil_comment_square: str = ":/icons/icons/cil-comment-square.png"
    cil_envelope_closed: str = ":/icons/icons/cil-envelope-closed.png"
    cil_envelope_letter: str = ":/icons/icons/cil-envelope-letter.png"
    cil_envelope_open: str = ":/icons/icons/cil-envelope-open.png"
    cil_phone: str = ":/icons/icons/cil-phone.png"
    cil_speech: str = ":/icons/icons/cil-speech.png"

    # ////// CIL ICONS - DEVICES
    cil_airplane_mode_off: str = ":/icons/icons/cil-airplane-mode-off.png"
    cil_airplane_mode: str = ":/icons/icons/cil-airplane-mode.png"
    cil_battery_0: str = ":/icons/icons/cil-battery-0.png"
    cil_battery_3: str = ":/icons/icons/cil-battery-3.png"
    cil_battery_5: str = ":/icons/icons/cil-battery-5.png"
    cil_battery_alert: str = ":/icons/icons/cil-battery-alert.png"
    cil_battery_slash: str = ":/icons/icons/cil-battery-slash.png"
    cil_bluetooth: str = ":/icons/icons/cil-bluetooth.png"
    cil_devices: str = ":/icons/icons/cil-devices.png"
    cil_ethernet: str = ":/icons/icons/cil-ethernet.png"
    cil_keyboard: str = ":/icons/icons/cil-keyboard.png"
    cil_laptop: str = ":/icons/icons/cil-laptop.png"
    cil_mobile_landscape: str = ":/icons/icons/cil-mobile-landscape.png"
    cil_mobile: str = ":/icons/icons/cil-mobile.png"
    cil_mouse: str = ":/icons/icons/cil-mouse.png"
    cil_screen_desktop: str = ":/icons/icons/cil-screen-desktop.png"
    cil_screen_smartphone: str = ":/icons/icons/cil-screen-smartphone.png"
    cil_wifi_signal_0: str = ":/icons/icons/cil-wifi-signal-0.png"
    cil_wifi_signal_1: str = ":/icons/icons/cil-wifi-signal-1.png"
    cil_wifi_signal_2: str = ":/icons/icons/cil-wifi-signal-2.png"
    cil_wifi_signal_4: str = ":/icons/icons/cil-wifi-signal-4.png"
    cil_wifi_signal_off: str = ":/icons/icons/cil-wifi-signal-off.png"

    # ////// CIL ICONS - MEDIA
    cil_3d: str = ":/icons/icons/cil-3d.png"
    cil_4k: str = ":/icons/icons/cil-4k.png"
    cil_camera_roll: str = ":/icons/icons/cil-camera-roll.png"
    cil_camera: str = ":/icons/icons/cil-camera.png"
    cil_cast: str = ":/icons/icons/cil-cast.png"
    cil_closed_captioning: str = ":/icons/icons/cil-closed-captioning.png"
    cil_hd: str = ":/icons/icons/cil-hd.png"
    cil_hdr: str = ":/icons/icons/cil-hdr.png"
    cil_headphones: str = ":/icons/icons/cil-headphones.png"
    cil_image_plus: str = ":/icons/icons/cil-image-plus.png"
    cil_image1: str = ":/icons/icons/cil-image1.png"
    cil_media_eject: str = ":/icons/icons/cil-media-eject.png"
    cil_media_pause: str = ":/icons/icons/cil-media-pause.png"
    cil_media_play: str = ":/icons/icons/cil-media-play.png"
    cil_media_skip_backward: str = ":/icons/icons/cil-media-skip-backward.png"
    cil_media_skip_forward: str = ":/icons/icons/cil-media-skip-forward.png"
    cil_media_step_backward: str = ":/icons/icons/cil-media-step-backward.png"
    cil_media_step_forward: str = ":/icons/icons/cil-media-step-forward.png"
    cil_media_stop: str = ":/icons/icons/cil-media-stop.png"
    cil_movie: str = ":/icons/icons/cil-movie.png"
    cil_speaker: str = ":/icons/icons/cil-speaker.png"
    cil_volume_high: str = ":/icons/icons/cil-volume-high.png"
    cil_volume_low: str = ":/icons/icons/cil-volume-low.png"
    cil_volume_off: str = ":/icons/icons/cil-volume-off.png"

    # ////// CIL ICONS - INTERFACE
    cil_alarm: str = ":/icons/icons/cil-alarm.png"
    cil_ban: str = ":/icons/icons/cil-ban.png"
    cil_bell: str = ":/icons/icons/cil-bell.png"
    cil_briefcase: str = ":/icons/icons/cil-briefcase.png"
    cil_browser: str = ":/icons/icons/cil-browser.png"
    cil_calendar_check: str = ":/icons/icons/cil-calendar-check.png"
    cil_clock: str = ":/icons/icons/cil-clock.png"
    cil_cursor_move: str = ":/icons/icons/cil-cursor-move.png"
    cil_cursor: str = ":/icons/icons/cil-cursor.png"
    cil_dialpad: str = ":/icons/icons/cil-dialpad.png"
    cil_exit_to_app: str = ":/icons/icons/cil-exit-to-app.png"
    cil_file: str = ":/icons/icons/cil-file.png"
    cil_folder_open: str = ":/icons/icons/cil-folder-open.png"
    cil_folder: str = ":/icons/icons/cil-folder.png"
    cil_home: str = ":/icons/icons/cil-home.png"
    cil_house: str = ":/icons/icons/cil-house.png"
    cil_menu: str = ":/icons/icons/cil-menu.png"
    cil_notes: str = ":/icons/icons/cil-notes.png"
    cil_options_horizontal: str = ":/icons/icons/cil-options-horizontal.png"
    cil_options: str = ":/icons/icons/cil-options.png"
    cil_settings: str = ":/icons/icons/cil-settings.png"
    cil_size_grip: str = ":/icons/icons/cil-size-grip.png"
    cil_task: str = ":/icons/icons/cil-task.png"
    cil_terminal: str = ":/icons/icons/cil-terminal.png"
    cil_window_maximize: str = ":/icons/icons/cil-window-maximize.png"
    cil_window_minimize: str = ":/icons/icons/cil-window-minimize.png"
    cil_window_restore: str = ":/icons/icons/cil-window-restore.png"

    # ////// CIL ICONS - USERS
    cil_account_logout: str = ":/icons/icons/cil-account-logout.png"
    cil_people: str = ":/icons/icons/cil-people.png"
    cil_user_female: str = ":/icons/icons/cil-user-female.png"
    cil_user_follow: str = ":/icons/icons/cil-user-follow.png"
    cil_user_unfollow: str = ":/icons/icons/cil-user-unfollow.png"
    cil_user: str = ":/icons/icons/cil-user.png"

    # ////// CIL ICONS - EMOTIONS
    cil_face_dead: str = ":/icons/icons/cil-face-dead.png"
    cil_frown: str = ":/icons/icons/cil-frown.png"
    cil_meh: str = ":/icons/icons/cil-meh.png"
    cil_mood_bad: str = ":/icons/icons/cil-mood-bad.png"
    cil_mood_good: str = ":/icons/icons/cil-mood-good.png"
    cil_mood_very_bad: str = ":/icons/icons/cil-mood-very-bad.png"
    cil_mood_very_good: str = ":/icons/icons/cil-mood-very-good.png"
    cil_smile: str = ":/icons/icons/cil-smile.png"

    # ////// CIL ICONS - MISC
    cil_av_timer: str = ":/icons/icons/cil-av-timer.png"
    cil_caret_bottom: str = ":/icons/icons/cil-caret-bottom.png"
    cil_caret_left: str = ":/icons/icons/cil-caret-left.png"
    cil_caret_right: str = ":/icons/icons/cil-caret-right.png"
    cil_caret_top: str = ":/icons/icons/cil-caret-top.png"
    cil_cart: str = ":/icons/icons/cil-cart.png"
    cil_chart_line: str = ":/icons/icons/cil-chart-line.png"
    cil_chart_pie: str = ":/icons/icons/cil-chart-pie.png"
    cil_chart: str = ":/icons/icons/cil-chart.png"
    cil_circle: str = ":/icons/icons/cil-circle.png"
    cil_clipboard: str = ":/icons/icons/cil-clipboard.png"
    cil_clone: str = ":/icons/icons/cil-clone.png"
    cil_cloud_download: str = ":/icons/icons/cil-cloud-download.png"
    cil_cloud_upload: str = ":/icons/icons/cil-cloud-upload.png"
    cil_cloudy: str = ":/icons/icons/cil-cloudy.png"
    cil_code: str = ":/icons/icons/cil-code.png"
    cil_coffee: str = ":/icons/icons/cil-coffee.png"
    cil_credit_card: str = ":/icons/icons/cil-credit-card.png"
    cil_cut: str = ":/icons/icons/cil-cut.png"
    cil_data_transfer_down: str = ":/icons/icons/cil-data-transfer-down.png"
    cil_data_transfer_up: str = ":/icons/icons/cil-data-transfer-up.png"
    cil_deaf: str = ":/icons/icons/cil-deaf.png"
    cil_description: str = ":/icons/icons/cil-description.png"
    cil_dog: str = ":/icons/icons/cil-dog.png"
    cil_door: str = ":/icons/icons/cil-door.png"
    cil_double_quote_sans_left: str = ":/icons/icons/cil-double-quote-sans-left.png"
    cil_double_quote_sans_right: str = ":/icons/icons/cil-double-quote-sans-right.png"
    cil_drop: str = ":/icons/icons/cil-drop.png"
    cil_equalizer: str = ":/icons/icons/cil-equalizer.png"
    cil_expand_down: str = ":/icons/icons/cil-expand-down.png"
    cil_expand_left: str = ":/icons/icons/cil-expand-left.png"
    cil_expand_right: str = ":/icons/icons/cil-expand-right.png"
    cil_expand_up: str = ":/icons/icons/cil-expand-up.png"
    cil_exposure: str = ":/icons/icons/cil-exposure.png"
    cil_external_link: str = ":/icons/icons/cil-external-link.png"
    cil_featured_playlist: str = ":/icons/icons/cil-featured-playlist.png"
    cil_find_in_page: str = ":/icons/icons/cil-find-in-page.png"
    cil_fingerprint: str = ":/icons/icons/cil-fingerprint.png"
    cil_fire: str = ":/icons/icons/cil-fire.png"
    cil_flip_to_back: str = ":/icons/icons/cil-flip-to-back.png"
    cil_gamepad: str = ":/icons/icons/cil-gamepad.png"
    cil_hand_point_down: str = ":/icons/icons/cil-hand-point-down.png"
    cil_hand_point_left: str = ":/icons/icons/cil-hand-point-left.png"
    cil_hand_point_right: str = ":/icons/icons/cil-hand-point-right.png"
    cil_hand_point_up: str = ":/icons/icons/cil-hand-point-up.png"
    cil_heart: str = ":/icons/icons/cil-heart.png"
    cil_highligt: str = ":/icons/icons/cil-highligt.png"
    cil_history: str = ":/icons/icons/cil-history.png"
    cil_infinity: str = ":/icons/icons/cil-infinity.png"
    cil_input_power: str = ":/icons/icons/cil-input-power.png"
    cil_input: str = ":/icons/icons/cil-input.png"
    cil_layers: str = ":/icons/icons/cil-layers.png"
    cil_level_down: str = ":/icons/icons/cil-level-down.png"
    cil_level_up: str = ":/icons/icons/cil-level-up.png"
    cil_library_add: str = ":/icons/icons/cil-library-add.png"
    cil_library: str = ":/icons/icons/cil-library.png"
    cil_lightbulb: str = ":/icons/icons/cil-lightbulb.png"
    cil_link_alt: str = ":/icons/icons/cil-link-alt.png"
    cil_link_broken: str = ":/icons/icons/cil-link-broken.png"
    cil_link: str = ":/icons/icons/cil-link.png"
    cil_location_pin: str = ":/icons/icons/cil-location-pin.png"
    cil_lock_locked: str = ":/icons/icons/cil-lock-locked.png"
    cil_lock_unlocked: str = ":/icons/icons/cil-lock-unlocked.png"
    cil_loop_1: str = ":/icons/icons/cil-loop-1.png"
    cil_loop_circular: str = ":/icons/icons/cil-loop-circular.png"
    cil_loop: str = ":/icons/icons/cil-loop.png"
    cil_low_vision: str = ":/icons/icons/cil-low-vision.png"
    cil_magnifying_glass: str = ":/icons/icons/cil-magnifying-glass.png"
    cil_map: str = ":/icons/icons/cil-map.png"
    cil_microphone: str = ":/icons/icons/cil-microphone.png"
    cil_move: str = ":/icons/icons/cil-move.png"
    cil_mug_tea: str = ":/icons/icons/cil-mug-tea.png"
    cil_mug: str = ":/icons/icons/cil-mug.png"
    cil_paint_bucket: str = ":/icons/icons/cil-paint-bucket.png"
    cil_paper_plane: str = ":/icons/icons/cil-paper-plane.png"
    cil_paperclip: str = ":/icons/icons/cil-paperclip.png"
    cil_paragraph: str = ":/icons/icons/cil-paragraph.png"
    cil_pen_alt: str = ":/icons/icons/cil-pen-alt.png"
    cil_pencil: str = ":/icons/icons/cil-pencil.png"
    cil_pin: str = ":/icons/icons/cil-pin.png"
    cil_power_standby: str = ":/icons/icons/cil-power-standby.png"
    cil_print: str = ":/icons/icons/cil-print.png"
    cil_rectangle: str = ":/icons/icons/cil-rectangle.png"
    cil_reload: str = ":/icons/icons/cil-reload.png"
    cil_remove: str = ":/icons/icons/cil-remove.png"
    cil_rss: str = ":/icons/icons/cil-rss.png"
    cil_satelite: str = ":/icons/icons/cil-satelite.png"
    cil_save: str = ":/icons/icons/cil-save.png"
    cil_share_boxed: str = ":/icons/icons/cil-share-boxed.png"
    cil_share: str = ":/icons/icons/cil-share.png"
    cil_signal_cellular_0: str = ":/icons/icons/cil-signal-cellular-0.png"
    cil_signal_cellular_3: str = ":/icons/icons/cil-signal-cellular-3.png"
    cil_speedometer: str = ":/icons/icons/cil-speedometer.png"
    cil_star: str = ":/icons/icons/cil-star.png"
    cil_tags: str = ":/icons/icons/cil-tags.png"
    cil_text_size: str = ":/icons/icons/cil-text-size.png"
    cil_text_square: str = ":/icons/icons/cil-text-square.png"
    cil_text: str = ":/icons/icons/cil-text.png"
    cil_thumb_down: str = ":/icons/icons/cil-thumb-down.png"
    cil_thumb_up: str = ":/icons/icons/cil-thumb-up.png"
    cil_transfer: str = ":/icons/icons/cil-transfer.png"
    cil_trash: str = ":/icons/icons/cil-trash.png"
    cil_triangle: str = ":/icons/icons/cil-triangle.png"
    cil_truck: str = ":/icons/icons/cil-truck.png"
    cil_vertical_align_bottom: str = ":/icons/icons/cil-vertical-align-bottom.png"
    cil_view_column: str = ":/icons/icons/cil-view-column.png"
    cil_view_module: str = ":/icons/icons/cil-view-module.png"
    cil_view_quilt: str = ":/icons/icons/cil-view-quilt.png"
    cil_view_stream: str = ":/icons/icons/cil-view-stream.png"
    cil_voice_over_record: str = ":/icons/icons/cil-voice-over-record.png"
    cil_wallet: str = ":/icons/icons/cil-wallet.png"
    cil_watch: str = ":/icons/icons/cil-watch.png"
    cil_wrap_text: str = ":/icons/icons/cil-wrap-text.png"
    cil_zoom_in: str = ":/icons/icons/cil-zoom-in.png"
    cil_zoom_out: str = ":/icons/icons/cil-zoom-out.png"



# ///////////////////////////////////////////////////////////////
# SERVICES.UI - UI services
# Project: ezqt_app
# ///////////////////////////////////////////////////////////////

"""UI services for reusable component creation."""

from __future__ import annotations

# ///////////////////////////////////////////////////////////////
# IMPORTS
# ///////////////////////////////////////////////////////////////
# Local imports
from .component_factory import (
    Fonts,
    SizePolicy,
    UiComponentFactory,
    get_ui_component_factory,
)
from .definitions_service import UiDefinitionsService
from .menu_service import MenuService
from .panel_service import PanelService
from .theme_service import ThemeService
from .ui_functions import UIFunctions
from .window_service import WindowService

# ///////////////////////////////////////////////////////////////
# PUBLIC API
# ///////////////////////////////////////////////////////////////
__all__ = [
    "UiComponentFactory",
    "get_ui_component_factory",
    "Fonts",
    "SizePolicy",
    "MenuService",
    "PanelService",
    "ThemeService",
    "WindowService",
    "UiDefinitionsService",
    "UIFunctions",
]
